@@ -6,24 +6,49 @@
 namespace code {
 	namespace x64 {
 
+		void nopOut(Output *to, Instr *instr) {
+			// Alias for xchg eax, eax
+			to->putByte(0x90);
+		}
+
 		void prologOut(Output *to, Instr *instr) {
-			// We generate:
-			// - push rbp
-			// - mov rbp, rsp
+			// There are two forms we need to consider:
+			Operand src = instr->src();
 
-			// push rbp
-			to->putByte(0x50 + 5);
-			// now the CFA offset is different
-			to->setFrameOffset(Offset::sPtr*2);
-			// We also saved RBP on the stack
-			to->markSaved(ptrFrame, -Offset::sPtr*2);
+			if (src.type() == opNone) {
+				// 1: if we have no parameter. This is used on Posix platforms. There, we generate a
+				// prolog with DWARF metadata as follows:
+				// - push rbp
+				// - mov rbp, rsp
 
-			// mov rbp, rsp
-			to->putByte(0x48);
-			to->putByte(0x89);
-			to->putByte(0xE5);
-			// now we use ebp as the CFA register, offset is the same
-			to->setFrameRegister(ptrFrame);
+				// push rbp
+				to->putByte(0x50 + 5);
+				// now the CFA offset is different
+				to->setFrameOffset(Offset::sPtr*2);
+				// We also saved RBP on the stack
+				to->markSaved(ptrFrame, -Offset::sPtr*2);
+
+				// mov rbp, rsp
+				to->putByte(0x48);
+				to->putByte(0x89);
+				to->putByte(0xE5);
+				// now we use ebp as the CFA register, offset is the same
+				to->setFrameRegister(ptrFrame);
+			} else if (src.type() == opConstant) {
+				// 2: if we have a constant as a parameter, we are on Windows. Then, generate code
+				// for allocating memory on the stack, and emit appropriate metadata:
+
+				// This is from 'sub' below:
+				ImmRegInstr op = {
+					opCode(0x83), 5,
+					opCode(0x81), 5,
+					opCode(0x29),
+					opCode(0x2B)
+				};
+				immRegInstr(to, op, ptrStack, src);
+				to->markFrameAlloc(src.offset());
+				to->markPrologEnd();
+			}
 		}
 
 		void epilogOut(Output *to, Instr *instr) {
@@ -38,10 +63,14 @@ namespace code {
 		}
 
 		void preserveOut(Output *to, Instr *instr) {
-			// Offset between the stack pointer and the CFA. This difference is due to us spilling
-			// RBP from the previous function, and the return address on the stack.
-			Size offset = Size::sPtr * 2;
-			to->markSaved(instr->src().reg(), instr->dest().offset() - offset);
+			if (instr->dest().type() == opNone) {
+				to->markSaved(instr->src().reg(), Offset());
+			} else {
+				// Offset between the stack pointer and the CFA. This difference is due to us
+				// spilling RBP from the previous function, and the return address on the stack.
+				Size offset = Size::sPtr * 2;
+				to->markSaved(instr->src().reg(), instr->dest().offset() - offset);
+			}
 		}
 
 		void locationOut(Output *, Instr *) {
@@ -115,15 +144,6 @@ namespace code {
 				return prefixOpCode(0xF3, 0x0F, op);
 		}
 
-		static bool stackPos(Operand &op) {
-			if (op.type() != opRegister)
-				return false;
-
-			// We can use a few bytes just above the top of the stack freely, at least on UNIX.
-			op = xRel(op.size(), ptrStack, -Offset(op.size()));
-			return true;
-		}
-
 		void movOut(Output *to, Instr *instr) {
 			// NOTE: There is a version of MOV that supports 64-bit immediate values. We could use
 			// that to avoid additional instructions in quite some cases!
@@ -145,22 +165,15 @@ namespace code {
 				// Two registers. This is the easy part!
 				modRm(to, fpOp(instr->src(), 0x10), rmNone, instr->dest(), instr->src());
 			} else if (fpSrc) {
-				// 'dest' is either in memory or a regular register.
+				// 'dest' is either in memory or in a regular register.
 				Operand dest = instr->dest();
-				bool spill = stackPos(dest);
-				modRm(to, fpOp(dest, 0x11), rmNone, instr->src(), dest);
-				if (spill) {
-					// Spilled to memory. Emit a regular 'mov' to load into the desired register.
-					immRegInstr(to, op8, op, instr->dest(), dest);
-				}
+				RmFlags flags = dest.size() == Size::sWord ? rmWide : rmNone;
+				modRm(to, prefixOpCode(0x66, 0x0F, 0x7E), flags, instr->src(), dest);
 			} else if (fpDest) {
 				// 'src' is either in memory or a regular register.
 				Operand src = instr->src();
-				if (stackPos(src)) {
-					// We need to spill to memory. Emit a regular 'mov'.
-					immRegInstr(to, op8, op, src, instr->src());
-				}
-				modRm(to, fpOp(src, 0x10), rmNone, instr->dest(), src);
+				RmFlags flags = src.size() == Size::sWord ? rmWide : rmNone;
+				modRm(to, prefixOpCode(0x66, 0x0F, 0x6E), flags, instr->dest(), src);
 			} else {
 				// Integer mov instruction.
 				immRegInstr(to, op8, op, instr->dest(), instr->src());
@@ -740,6 +753,7 @@ namespace code {
 		typedef void (*OutputFn)(Output *to, Instr *instr);
 
 		const OpEntry<OutputFn> outputMap[] = {
+			OUTPUT(nop),
 			OUTPUT(push),
 			OUTPUT(pop),
 			OUTPUT(mov),
