@@ -2,9 +2,10 @@
 #include "Layout.h"
 #include "Exception.h"
 #include "Asm.h"
+#include "Arena.h"
 #include "../Binary.h"
 #include "../Layout.h"
-#include "../PosixEh/FnState.h"
+#include "../FnState.h"
 
 namespace code {
 	namespace x64 {
@@ -27,7 +28,7 @@ namespace code {
 		};
 
 
-		Layout::Layout() {}
+		Layout::Layout(const Arena *arena) : arena(arena) {}
 
 		void Layout::before(Listing *dest, Listing *src) {
 			// Initialize some state.
@@ -35,7 +36,7 @@ namespace code {
 			usingEH = src->exceptionAware();
 
 			// Compute the layout of all parameters.
-			params = new (this) Params();
+			params = arena->createParams(src->member);
 			params->result(src->result);
 			Array<Var> *p = src->allParams();
 			for (Nat i = 0; i < p->count(); i++) {
@@ -44,15 +45,11 @@ namespace code {
 
 			// Figure out which registers we need to spill.
 			toPreserve = allUsedRegs(src);
-			for (size_t i = 0; i < fnDirtyCount; i++)
-				toPreserve->remove(fnDirtyRegs[i]);
+			for (RegSet::Iter i = arena->dirtyRegs->begin(), end = arena->dirtyRegs->end(); i != end; ++i)
+				toPreserve->remove(*i);
 
 			// Compute the layout.
-			Nat spilled = toPreserve->count();
-			if (params->result().memoryRegister() != noReg)
-				spilled++; // The hidden parameter needs to be spilled!
-
-			layout = code::x64::layout(src, params, spilled);
+			layout = computeLayout(src, params, toPreserve->count());
 
 			// Initialize the 'activated' array.
 			Array<Var> *vars = src->allVars();
@@ -116,7 +113,7 @@ namespace code {
 			for (Nat i = 0; i < activeBlocks->count(); i++) {
 				const ActiveBlock &a = activeBlocks->at(i);
 				*dest << lblOffset(a.pos);
-				*dest << dat(natConst(code::eh::encodeFnState(a.block.key(), a.activated)));
+				*dest << dat(natConst(code::encodeFnState(a.block.key(), a.activated)));
 			}
 
 			// Table size.
@@ -137,7 +134,15 @@ namespace code {
 			return xRel(size, ptrFrame, layout->at(v.key()) + op.offset());
 		}
 
+		static bool spillAlways(const Param &, const Offset &) {
+			return true;
+		}
+
 		void Layout::spillParams(Listing *dest) {
+			spillParams(dest, &spillAlways);
+		}
+
+		void Layout::spillParams(Listing *dest, SpillPredicate p) {
 			Array<Var> *all = dest->allParams();
 
 			for (Nat i = 0; i < params->registerCount(); i++) {
@@ -145,13 +150,19 @@ namespace code {
 				if (info.size() == Size())
 					continue; // Not used.
 				if (info.id() == Param::returnId()) {
-					*dest << mov(ptrRel(ptrFrame, resultParam()), params->registerSrc(i));
+					Offset to = resultParam();
+					if ((*p)(info, to))
+						*dest << mov(ptrRel(ptrFrame, resultParam()), params->registerSrc(i));
 					continue;
 				}
 
 				Offset to = layout->at(all->at(info.id()).key());
 				to += Offset(info.offset());
 
+				// Skip if predicate says so.
+				if (!(*p)(info, to))
+					continue;
+
 				Size size(info.size());
 				Reg r = asSize(params->registerSrc(i), size);
 				if (r == noReg) {
@@ -166,27 +177,7 @@ namespace code {
 		void Layout::prologTfm(Listing *dest, Listing *src, Nat line) {
 			// Generate the prolog. Generates push and mov to set up a basic stack frame. Also emits
 			// proper unwind data.
-			*dest << prolog();
-
-			// *dest << push(ptrFrame);
-			// *dest << mov(ptrFrame, ptrStack);
-
-			// Allocate stack space.
-			if (layout->last() != Offset())
-				*dest << sub(ptrStack, ptrConst(layout->last()));
-
-			// Keep track of offsets.
-			Offset offset = -Offset::sPtr;
-
-			// Save registers we need to preserve.
-			for (RegSet::Iter i = toPreserve->begin(); i != toPreserve->end(); ++i) {
-				*dest << mov(ptrRel(ptrFrame, offset), asSize(i.v(), Size::sPtr));
-				*dest << preserve(ptrRel(ptrFrame, offset), asSize(i.v(), Size::sPtr));
-				offset -= Offset::sPtr;
-			}
-
-			// Spill parameters to the stack.
-			spillParams(dest);
+			emitProlog(dest);
 
 			// Initialize the root block.
 			initBlock(dest, dest->root(), rax);
@@ -205,18 +196,7 @@ namespace code {
 			}
 			block = oldBlock;
 
-			// Restore preserved registers.
-			Offset offset = -Offset::sPtr;
-			for (RegSet::Iter i = toPreserve->begin(); i != toPreserve->end(); ++i) {
-				*dest << mov(asSize(i.v(), Size::sPtr), ptrRel(ptrFrame, offset));
-				offset -= Offset::sPtr;
-			}
-
-			// The "epilog" pseudo-op generates a LEAVE instruction, which corresponds to these instructions:
-			// *dest << mov(ptrStack, ptrFrame);
-			// *dest << pop(ptrFrame);
-
-			*dest << code::epilog();
+			emitEpilog(dest);
 		}
 
 		void Layout::beginBlockTfm(Listing *dest, Listing *src, Nat line) {
@@ -265,17 +245,15 @@ namespace code {
 
 			// We only need to update the block id if this impacts exception handling.
 			if (src->freeOpt(var) & freeOnException) {
+				// We might need a nop here if the last was a call.
+				padCallWithNop(dest);
+
 				Label lbl = dest->label();
 				*dest << lbl;
 				activeBlocks->push(ActiveBlock(block, activationId, lbl));
 			}
 		}
 
-		Offset Layout::resultParam() {
-			Nat count = 1 + toPreserve->count();
-			return -(Offset::sPtr * count);
-		}
-
 		// Zero the memory of a variable. 'initReg' should be true if we need to set <reg> to 0
 		// before using it as our zero. 'initReg' will be set to false, so that it is easy to use
 		// zeroVar in a loop, causing only the first invocation to emit '<reg> := 0'.
@@ -286,6 +264,8 @@ namespace code {
 
 			reg = asSize(reg, Size::sLong);
 
+			// Note: We could use: and <dest>, 0 - that can be encoded compactly and does not
+			// require additional registers! This is what some versions of MSVC does.
 			if (initReg) {
 				*dest << bxor(reg, reg);
 				initReg = false;
@@ -309,7 +289,7 @@ namespace code {
 			}
 		}
 
-		void Layout::initBlock(Listing *dest, Block init, Reg reg) {
+		void Layout::initBlock(Listing *dest, Block init, Operand tempSpace) {
 			if (block != dest->parent(init)) {
 				Str *msg = TO_S(engine(), S("Can not begin ") << init << S(" unless the current is ")
 								<< dest->parent(init) << S(". Current is ") << block);
@@ -318,26 +298,53 @@ namespace code {
 
 			block = init;
 
-			Bool initReg = true;
-			Bool restoreReg = reg == noReg;
-			if (restoreReg) {
-				reg = eax;
-				*dest << push(eax);
+			Array<Var> *vars = dest->allVars(init);
+
+			// Figure out which register to use for zero:
+			Reg zeroReg = noReg;
+			Bool initZeroReg = true;
+			Bool pushed = false;
+
+			if (tempSpace.type() == opRegister) {
+				zeroReg = tempSpace.reg();
+			} else if (tempSpace != Operand()) {
+				// Spill register to memory:
+				if (vars->any()) {
+					*dest << mov(tempSpace, ptrA);
+					zeroReg = ptrA;
+				}
+			} else {
+				// Note: Not viable on Windows, but preprocessing step ensures we never get here.
+				if (vars->any()) {
+					zeroReg = ptrA;
+					*dest << push(ptrA);
+					pushed = true;
+				}
 			}
 
-			Array<Var> *vars = dest->allVars(init);
 			// Go in reverse to make linear accesses in memory when we're using big variables.
 			for (Nat i = vars->count(); i > 0; i--) {
 				Var v = vars->at(i - 1);
 
-				if (!dest->isParam(v))
-					zeroVar(dest, layout->at(v.key()), v.size(), reg, initReg);
+				// Don't initialize parameters or variables that we don't need to initialize.
+				if (!dest->isParam(v) && (dest->freeOpt(v) & freeNoInit) == 0)
+					zeroVar(dest, layout->at(v.key()), v.size(), zeroReg, initZeroReg);
 			}
 
-			if (restoreReg)
-				*dest << pop(eax);
+			// Restore any register if necessary:
+			if (pushed || !initZeroReg) {
+				if (tempSpace.type() == opRegister) {
+					// Nothing to do.
+				} else if (tempSpace != Operand()) {
+					*dest << mov(ptrA, tempSpace);
+				} else {
+					*dest << pop(ptrA);
+				}
+			}
 
 			if (usingEH) {
+				padCallWithNop(dest);
+
 				// Remember where the block started.
 				Label lbl = dest->label();
 				*dest << lbl;
@@ -345,38 +352,11 @@ namespace code {
 			}
 		}
 
-		static void saveResult(Listing *dest, const Result &result) {
-			if (result.memoryRegister() != noReg) {
-				// We need to keep the stack aligned to 16 bits.
-				*dest << push(result.memoryRegister());
-				*dest << push(result.memoryRegister());
-			} else if (result.registerCount() > 0) {
-				Size sz = Size::sPtr * roundUp(result.registerCount(), Nat(2));
-				*dest << sub(ptrStack, ptrConst(sz));
-				for (Nat i = 0; i < result.registerCount(); i++) {
-					*dest << mov(ptrRel(ptrStack, Offset::sPtr * i), asSize(result.registerAt(i), Size::sPtr));
-				}
-			}
-		}
-
-		static void restoreResult(Listing *dest, const Result &result) {
-			if (result.memoryRegister() != noReg) {
-				*dest << pop(result.memoryRegister());
-				*dest << pop(result.memoryRegister());
-			} else if (result.registerCount() > 0) {
-				Size sz = Size::sPtr * roundUp(result.registerCount(), Nat(2));
-				for (Nat i = 0; i < result.registerCount(); i++) {
-					*dest << mov(asSize(result.registerAt(i), Size::sPtr), ptrRel(ptrStack, Offset::sPtr * i));
-				}
-				*dest << add(ptrStack, ptrConst(sz));
-			}
-		}
-
 		void Layout::destroyBlock(Listing *dest, Block destroy, Bool preserveRax, Bool table) {
 			if (destroy != block)
 				throw new (this) BlockEndError();
 
-			Bool pushedResult = false;
+			Bool savedResult = false;
 			Array<Var> *vars = dest->allVars(destroy);
 			// Destroy in reverse order.
 			for (Nat i = vars->count(); i > 0; i--) {
@@ -390,26 +370,29 @@ namespace code {
 					if (activated->at(v.key()) > activationId)
 						continue;
 
-					if (preserveRax && !pushedResult) {
-						saveResult(dest, params->result());
-						pushedResult = true;
+					if (preserveRax && !savedResult) {
+						saveResult(dest);
+						savedResult = true;
 					}
 
+					Reg firstParam = params->registerSrc(0);
+
+					// Note: code in WindowsLayout.cpp ensures that we have enough shadow space here.
 					if (when & freeIndirection) {
 						if (when & freePtr) {
-							*dest << mov(ptrDi, resolve(dest, v, Size::sPtr));
+							*dest << mov(firstParam, resolve(dest, v, Size::sPtr));
 							*dest << call(dtor, Size());
 						} else {
-							*dest << mov(ptrDi, resolve(dest, v, Size::sPtr));
-							*dest << mov(asSize(ptrDi, v.size()), xRel(v.size(), ptrDi, Offset()));
+							*dest << mov(firstParam, resolve(dest, v, Size::sPtr));
+							*dest << mov(asSize(firstParam, v.size()), xRel(v.size(), firstParam, Offset()));
 							*dest << call(dtor, Size());
 						}
 					} else {
 						if (when & freePtr) {
-							*dest << lea(ptrDi, resolve(dest, v));
+							*dest << lea(firstParam, resolve(dest, v));
 							*dest << call(dtor, Size());
 						} else {
-							*dest << mov(asSize(ptrDi, v.size()), resolve(dest, v));
+							*dest << mov(asSize(firstParam, v.size()), resolve(dest, v));
 							*dest << call(dtor, Size());
 						}
 					}
@@ -417,19 +400,22 @@ namespace code {
 				}
 			}
 
-			if (pushedResult) {
-				restoreResult(dest, params->result());
+			if (savedResult) {
+				restoreResult(dest);
 			}
 
 			block = dest->parent(block);
 			if (usingEH && table) {
+				padCallWithNop(dest);
+
 				Label lbl = dest->label();
 				*dest << lbl;
 				activeBlocks->push(ActiveBlock(block, activationId, lbl));
 			}
 		}
 
-		// Memcpy using mov instructions.
+		// Memcpy using mov instructions. Note: rdx does not need to be preserved,
+		// and does never contain a part of the result on any of the supported ABIs.
 		static void movMemcpy(Listing *to, Reg dest, Reg src, Size size) {
 			Nat total = size.size64();
 			Nat offset = 0;
@@ -450,36 +436,41 @@ namespace code {
 			}
 		}
 
-		// Put the return value into registers. Assumes ptrSi contains a pointer to the struct to be returned.
-		static void returnSimple(Listing *dest, const Result &result, SimpleDesc *type, Offset resultParam) {
+		// Put the return value into registers. Assumes 'srcPtr' contains a pointer to the struct to
+		// be returned. 'srcPtr' can not be 'rdx'.
+		static void returnSimple(Listing *dest, const Result &result, SimpleDesc *type,
+								Reg srcPtr, Offset resultParam) {
+
+			assert(!same(srcPtr, rdx));
+
 			if (result.memoryRegister() != noReg) {
 				*dest << mov(result.memoryRegister(), ptrRel(ptrFrame, resultParam));
 				// Inline a memcpy using mov instructions.
-				movMemcpy(dest, result.memoryRegister(), ptrSi, type->size());
+				movMemcpy(dest, result.memoryRegister(), srcPtr, type->size());
 			} else {
 				for (Nat i = 0; i < result.registerCount(); i++) {
 					Reg r = result.registerAt(i);
-					*dest << mov(r, xRel(size(r), ptrSi, Offset(result.registerOffset(i))));
+					*dest << mov(r, xRel(size(r), srcPtr, Offset(result.registerOffset(i))));
 				}
 			}
 		}
 
-		static void returnPrimitive(Listing *dest, PrimitiveDesc *p, const Operand &value) {
+		static void returnPrimitive(Listing *dest, PrimitiveDesc *p, const Operand &value, Reg target) {
 			switch (p->v.kind()) {
 			case primitive::none:
 				break;
 			case primitive::integer:
 			case primitive::pointer:
-				if (value.type() == opRegister && same(value.reg(), ptrA)) {
+				if (value.type() == opRegister && same(value.reg(), target)) {
 					// Already at the correct place!
 				} else {
 					// A simple 'mov' is enough!
-					*dest << mov(asSize(ptrA, value.size()), value);
+					*dest << mov(asSize(target, value.size()), value);
 				}
 				break;
 			case primitive::real:
 				// A simple 'mov' will do!
-				*dest << mov(asSize(xmm0, value.size()), value);
+				*dest << mov(asSize(target, value.size()), value);
 				break;
 			}
 		}
@@ -497,26 +488,39 @@ namespace code {
 
 			// Handle the return value.
 			if (PrimitiveDesc *p = as<PrimitiveDesc>(src->result)) {
-				returnPrimitive(dest, p, value);
+				if (params->result().registerCount() > 0)
+					returnPrimitive(dest, p, value, params->result().registerAt(0));
 			} else if (ComplexDesc *c = as<ComplexDesc>(src->result)) {
+				// Note: Windows-specific pre-processing ensures we have shadow space here.
 				// Call the copy-ctor.
-				*dest << lea(ptrSi, value);
-				*dest << mov(ptrDi, ptrRel(ptrFrame, resultParam()));
+				*dest << lea(params->registerSrc(1), value);
+				*dest << mov(params->registerSrc(0), ptrRel(ptrFrame, resultParam()));
 				*dest << call(c->ctor, Size());
 				// Set 'rax' to the address of the return value.
 				*dest << mov(ptrA, ptrRel(ptrFrame, resultParam()));
 			} else if (SimpleDesc *s = as<SimpleDesc>(src->result)) {
-				*dest << lea(ptrSi, value);
-				returnSimple(dest, params->result(), s, resultParam());
+				if (params->result().memoryRegister() != noReg) {
+					// Just copy using memcpy. We also need to set 'ptrA' accordingly.
+					// Note: ptrC is always safe to use in this context, both on Windows and Posix.
+					*dest << lea(ptrC, value);
+					*dest << mov(ptrA, ptrRel(ptrFrame, resultParam()));
+					movMemcpy(dest, ptrA, ptrC, s->size());
+				} else {
+					Reg r = params->registerSrc(0);
+					*dest << lea(r, value);
+					returnSimple(dest, params->result(), s, r, resultParam());
+				}
 			} else {
 				assert(false);
 			}
 
+			// Note: We could avoid some pain while calling dtors by placing code "inside" the
+			// epilog codegen. For example, we could load 'resultParam' just before the epilog.
 			epilogTfm(dest, src, line);
 			*dest << ret(Size()); // We will not analyze registers anymore, Size() is fine.
 		}
 
-		static void returnPrimitiveRef(Listing *dest, PrimitiveDesc *p, const Operand &value) {
+		static void returnPrimitiveRef(Listing *dest, PrimitiveDesc *p, const Operand &value, Reg target) {
 			Size s(p->v.size());
 			switch (p->v.kind()) {
 			case primitive::none:
@@ -524,12 +528,13 @@ namespace code {
 			case primitive::integer:
 			case primitive::pointer:
 				// Always two 'mov'.
-				*dest << mov(ptrA, value);
-				*dest << mov(asSize(ptrA, s), xRel(s, ptrA, Offset()));
+				*dest << mov(asSize(target, Size::sPtr), value);
+				*dest << mov(asSize(target, s), xRel(s, target, Offset()));
 				break;
 			case primitive::real:
+				// Note: ptrA is safe as temporary here.
 				*dest << mov(ptrA, value);
-				*dest << mov(asSize(xmm0, s), xRel(s, ptrA, Offset()));
+				*dest << mov(asSize(target, s), xRel(s, ptrA, Offset()));
 				break;
 			}
 		}
@@ -539,110 +544,36 @@ namespace code {
 
 			// Handle the return value.
 			if (PrimitiveDesc *p = as<PrimitiveDesc>(src->result)) {
-				returnPrimitiveRef(dest, p, value);
+				if (params->result().registerCount() > 0)
+					returnPrimitiveRef(dest, p, value, params->result().registerAt(0));
 			} else if (ComplexDesc *c = as<ComplexDesc>(src->result)) {
 				// Call the copy-ctor.
-				*dest << mov(ptrSi, value);
-				*dest << mov(ptrDi, ptrRel(ptrFrame, resultParam()));
+				*dest << mov(params->registerSrc(1), value);
+				*dest << mov(params->registerSrc(0), ptrRel(ptrFrame, resultParam()));
 				*dest << call(c->ctor, Size());
 				// Set 'rax' to the address of the return value.
 				*dest << mov(ptrA, ptrRel(ptrFrame, resultParam()));
 			} else if (SimpleDesc *s = as<SimpleDesc>(src->result)) {
-				*dest << mov(ptrSi, value);
-				returnSimple(dest, params->result(), s, resultParam());
+				if (params->result().memoryRegister() != noReg) {
+					// Just copy using memcpy. We also need to set 'ptrA' accordingly.
+					// Note: ptrC is always safe to use here!
+					*dest << mov(ptrC, value);
+					*dest << mov(ptrA, ptrRel(ptrFrame, resultParam()));
+					movMemcpy(dest, ptrA, ptrC, s->size());
+				} else {
+					Reg r = params->registerSrc(0);
+					*dest << mov(r, value);
+					returnSimple(dest, params->result(), s, r, resultParam());
+				}
 			} else {
 				assert(false);
 			}
 
+			// Note: We could avoid some pain while calling dtors by placing code "inside" the
+			// epilog codegen. For example, we could load 'resultParam' just before the epilog.
 			epilogTfm(dest, src, line);
 			*dest << ret(Size()); // We will not analyze registers anymore, Size() is fine.
 		}
 
-
-		static Size spillParams(Array<Offset> *out, Listing *src, Params *params, Offset varOffset) {
-			// NOTE: We could avoid spilling primitives to memory, as we do not generally use those
-			// registers from code generated for the generic platform. However, we would need to
-			// save as soon as we perform a function call anyway, which is probably more expensive
-			// than just spilling them unconditionally in the function prolog.
-
-			Array<Var> *all = src->allParams();
-
-			{
-				// Start by computing the locations of parameters passed on the stack.
-				Offset stackOffset = Offset::sPtr * 2;
-				for (Nat i = 0; i < params->stackCount(); i++) {
-					Nat paramId = params->stackParam(i).id(); // Should never be 'Param::returnId'
-					Var var = all->at(paramId);
-
-					out->at(var.key()) = stackOffset;
-					stackOffset = (stackOffset + var.size().aligned()).alignAs(Size::sPtr);
-				}
-			}
-
-			Size used;
-			// Then, compute where to spill the remaining registers.
-			for (Nat i = 0; i < all->count(); i++) {
-				Var var = all->at(i);
-				Offset &to = out->at(var.key());
-
-				// Already computed -> no need to spill.
-				if (to != Offset())
-					continue;
-
-				// Try to squeeze the preserved registers as tightly as possible while keeping alignment.
-				// This is more concentrated than when pushing registers to the stack. On the stack, all
-				// parameters are aligned to 8 bytes, while we do not need that here. We only need to keep
-				// the natural alignment of the parameters to keep the CPU happy.
-				Size sz = var.size();
-				if (src->freeOpt(var) & freeIndirection)
-					sz = Size::sPtr;
-
-				// However, since we don't want to use multiple instructions to spill a single
-				// parameter, we increase the size of some parameters a bit.
-				if (asSize(ptrA, sz) == noReg)
-					sz = sz + Size::sInt.alignment();
-
-				used = (used + sz).alignedAs(sz);
-				to = -(varOffset + used);
-			}
-
-			return used.aligned();
-		}
-
-		Array<Offset> *layout(Listing *src, Params *params, Nat spilled) {
-			Array<Offset> *result = code::layout(src);
-
-			// Saved registers and other things:
-			Offset varOffset = Offset::sPtr * spilled;
-
-			// Figure out which variables we need to spill into memory.
-			varOffset += spillParams(result, src, params, varOffset);
-			varOffset = varOffset.alignAs(Size::sPtr);
-
-			// Update the layout of the other variables according to the size we needed for
-			// parameters and spilled registers.
-			Array<Var> *all = src->allVars();
-			for (Nat i = 0; i < all->count(); i++) {
-				Var var = all->at(i);
-				Nat id = var.key();
-
-				if (!src->isParam(var)) {
-					Size s = var.size();
-					if (src->freeOpt(var) & freeIndirection)
-						s = Size::sPtr;
-					result->at(id) = -(result->at(id) + s.aligned() + varOffset);
-				}
-			}
-
-			result->last() += varOffset;
-			if (result->last().v64() & 0xF) {
-				// We need to be aligned to 64 bits. Otherwise the SIMD-operations used widely on
-				// X86-64 will not work properly.
-				result->last() += Offset::sPtr;
-			}
-
-			return result;
-		}
-
 	}
 }
