@@ -44,6 +44,9 @@ namespace storm {
 			// Add a decorator.
 			void STORM_FN decorate(SrcName *decorator);
 
+			// Make the class final.
+			void STORM_FN makeFinal();
+
 		protected:
 			// Load the body lazily.
 			virtual Bool STORM_FN loadAll();
@@ -151,6 +154,9 @@ namespace storm {
 			// Called after 'items' are loaded but before 'wraps' are loaded.
 			virtual void STORM_FN prepareWraps();
 
+			// Called when all items are loaded.
+			virtual void STORM_FN finished();
+
 			// Owning class.
 			Class *owner;
 
