@@ -9,6 +9,7 @@
 #include "Core/Thread.h"
 #include "Core/Str.h"
 #include "Core/Map.h"
+#include "Core/Set.h"
 #include "Core/StrBuf.h"
 #include "Core/Handle.h"
 #include "Core/Io/Utf8Text.h"
@@ -72,63 +73,14 @@ namespace storm {
 		currentEngine = null;
 	}
 
-	static void findThreads(Array<NamedThread *> *result, Named *root) {
-		if (NamedThread *t = as<NamedThread>(root)) {
-			result->push(t);
-			return;
-		}
-
-		NameSet *search = as<NameSet>(root);
-		if (!search)
-			return;
-
-		for (NameSet::Iter i = search->begin(), e = search->end(); i != e; ++i) {
-			findThreads(result, i.v());
-		}
-	}
-
-	static void outputThread(StrBuf *output, const os::Thread &thread, Array<NamedThread *> *names) {
-		// Find the name of this thread (does not need to be very fast).
-		for (Nat i = 0; i < names->count(); i++) {
-			if (names->at(i)->thread()->sameAs(thread)) {
-				*output << names->at(i)->identifier();
-				return;
-			}
-		}
-
-		*output << S("Thread ") << hex(thread.id());
-	}
-
-	void Engine::threadSummary() {
+	vector<os::Thread> Engine::allThreads() {
 		vector<os::Thread> threads = threadGroup.threads();
 
 		os::Thread compiler = world.threads[0]->thread();
 		if (std::find(threads.begin(), threads.end(), compiler) == threads.end())
 			threads.insert(threads.begin(), compiler);
 
-		vector<vector<::StackTrace>> traces = os::stackTraces(threads);
-
-		Array<NamedThread *> *threadNames = new (*this) Array<NamedThread *>();
-		findThreads(threadNames, package());
-
-		StrBuf *output = new (*this) StrBuf();
-
-		for (size_t t = 0; t < threads.size(); t++) {
-			const os::Thread &thread = threads[t];
-			vector<::StackTrace> &trace = traces[t];
-
-			outputThread(output, thread, threadNames);
-			*output << L":\n";
-
-			Indent z(output);
-
-			for (size_t u = 0; u < trace.size(); u++) {
-				*output << S("UThread ") << u << S(":\n");
-				*output << format(trace[u]).c_str();
-			}
-		}
-
-		stdOut()->writeLine(output->toS());
+		return threads;
 	}
 
 	// Starts at -1 so that the first Engine will get id=0.
@@ -462,6 +414,10 @@ namespace storm {
 			return FNREF(MapBase::atRawValue);
 		case builtin::mapAtClass:
 			return FNREF(MapBase::atRawClass);
+		case builtin::mapGetUnsafe:
+			return FNREF(MapBase::getUnsafeRaw);
+		case builtin::setGetUnsafe:
+			return FNREF(SetBase::getUnsafeRaw);
 		case builtin::enumToS:
 			return FNREF(Enum::toString);
 		case builtin::futurePost:
@@ -570,6 +526,14 @@ namespace storm {
 		o.stdError = to;
 	}
 
+	Array<Str *> *Engine::argv() {
+		return atomicRead(o.argv);
+	}
+
+	void Engine::argv(Array<Str *> *val) {
+		atomicWrite(o.argv, val);
+	}
+
 	code::TypeDesc *Engine::ptrDesc() {
 		if (!o.ptrDesc)
 			o.ptrDesc = code::ptrDesc(*this);
