@@ -469,22 +469,32 @@ namespace storm {
 	}
 
 	void Function::asyncLocalCall(CodeGen *to, Array<code::Operand> *params, CodeResult *result, CodeResult *id) {
-		asyncCall(to, params, result, id, false);
+		asyncCall(to, params, code::Operand(), result, id, false);
 	}
 
 	void Function::asyncThreadCall(CodeGen *to, Array<code::Operand> *params, CodeResult *result) {
-		asyncThreadCall(to, params, result, null);
+		asyncThreadCall(to, params, result, code::Operand(), null);
+	}
+
+	void Function::asyncThreadCall(CodeGen *to, Array<code::Operand> *params, CodeResult *result, code::Operand thread) {
+		asyncThreadCall(to, params, result, thread, null);
 	}
 
 	void Function::asyncThreadCall(CodeGen *to, Array<code::Operand> *params, CodeResult *result, CodeResult *id) {
-		asyncCall(to, params, result, id, true);
+		asyncThreadCall(to, params, result, code::Operand(), id);
 	}
 
-	void Function::asyncCall(CodeGen *to, Array<code::Operand> *params, CodeResult *result, CodeResult *id, bool copy) {
+	void Function::asyncThreadCall(CodeGen *to, Array<code::Operand> *params, CodeResult *result, code::Operand thread, CodeResult *id) {
+		asyncCall(to, params, thread, result, id, true);
+	}
+
+	void Function::asyncCall(CodeGen *to, Array<code::Operand> *params, code::Operand thread,
+							CodeResult *result, CodeResult *id, bool copy) {
 		using namespace code;
 
 		// Figure out what thread to pass to the spawn function.
-		code::Operand thread = ptrConst(0);
+		if (thread.empty())
+			thread = ptrConst(0);
 		if (runOn().state != RunOn::any)
 			thread = findThread(to, params);
 
@@ -500,6 +510,8 @@ namespace storm {
 
 		// Create the result object.
 		Type *futureT = wrapFuture(e, this->result).type;
+		// Not technically necessary to do before calling 'safeLocation', but we do it anyway for future-proofing.
+		Bool resultNeeded = result->needed();
 		code::Var resultPos = result->safeLocation(sub, thisPtr(futureT));
 		allocObject(sub, futureT->defaultCtor(), new (this) Array<Operand>(), resultPos);
 		result->created(sub);
@@ -533,6 +545,17 @@ namespace storm {
 			*to->l << fnCall(e.ref(builtin::spawnFuture), false);
 		}
 
+		// If the result was not used, call 'detach' on the future:
+		if (!resultNeeded) {
+			Function *detachFn = as<Function>(futureT->find(S("detach"), Value(futureT), Scope()));
+			if (detachFn) {
+				*to->l << fnParam(ptrDesc(e), resultPos);
+				*to->l << fnCall(detachFn->ref(), true);
+			} else {
+				WARNING(L"Failed to find Future::detach. Will not call it automatically!");
+			}
+		}
+
 		// Now, we're done!
 		*to->l << end(sub->block);
 	}
