@@ -14,10 +14,15 @@ namespace storm {
 		// Quick print line function.
 		void STORM_FN print(Str *s) ON(Compiler);
 
-		// Get the stdout text stream.
+		// Get standard IO streams.
 		TextInput *STORM_FN stdIn(EnginePtr e) ON(Compiler);
 		TextOutput *STORM_FN stdOut(EnginePtr e) ON(Compiler);
 		TextOutput *STORM_FN stdError(EnginePtr e) ON(Compiler);
 
+		// Set standard IO streams.
+		void STORM_FN stdIn(EnginePtr e, TextInput *s) ON(Compiler);
+		void STORM_FN stdOut(EnginePtr e, TextOutput *s) ON(Compiler);
+		void STORM_FN stdError(EnginePtr e, TextOutput *s) ON(Compiler);
+
 	}
 }
