@@ -32,6 +32,25 @@ namespace storm {
 		Type *contents;
 	};
 
+	/**
+	 * Iterator type for queues.
+	 */
+	class QueueIterType : public Type {
+		STORM_CLASS;
+	public:
+		// Create.
+		QueueIterType(Type *param);
+
+	protected:
+		// Lazy loading.
+		virtual Bool STORM_FN loadAll();
+
+	private:
+		// Content type.
+		Type *contents;
+	};
+
+
 	Bool STORM_FN isQueue(Value v);
 	Value STORM_FN unwrapQueue(Value v);
 	Value STORM_FN wrapQueue(Value v);
