@@ -75,13 +75,21 @@ namespace storm {
 		return distance;
 	}
 
+	MAYBE(SimplePart *) SimplePart::nextOption() const {
+		return null;
+	}
+
+	Bool SimplePart::scopeParam(Nat id) const {
+		return false;
+	}
+
 	Bool SimplePart::visible(Named *candidate, Scope source) const {
 		return candidate->visibleFrom(source);
 	}
 
 	static void outputPart(StrBuf *to, const Value &v) {
 		if (v.type)
-			v.type->path()->toS(to);
+			v.type->safePath()->toS(to);
 		else
 			*to << S("void");
 		if (v.ref)
