@@ -44,7 +44,7 @@ namespace storm {
 		engine(RootObject::engine()),
 		myGcType(null),
 		tHandle(null),
-		typeFlags(flags & ~typeCpp) {
+		myTypeFlags(flags & ~typeCpp) {
 
 		init(null);
 	}
@@ -54,7 +54,7 @@ namespace storm {
 		engine(RootObject::engine()),
 		myGcType(null),
 		tHandle(null),
-		typeFlags(flags & ~typeCpp) {
+		myTypeFlags(flags & ~typeCpp) {
 
 		init(null);
 	}
@@ -64,7 +64,7 @@ namespace storm {
 		engine(RootObject::engine()),
 		myGcType(null),
 		tHandle(null),
-		typeFlags(flags & ~typeCpp) {
+		myTypeFlags(flags & ~typeCpp) {
 
 		this->pos = pos;
 		init(null);
@@ -75,7 +75,7 @@ namespace storm {
 		engine(RootObject::engine()),
 		myGcType(null),
 		tHandle(null),
-		typeFlags(flags & ~typeCpp) {
+		myTypeFlags(flags & ~typeCpp) {
 
 		this->pos = pos;
 		init(null);
@@ -86,7 +86,7 @@ namespace storm {
 		engine(RootObject::engine()),
 		myGcType(null),
 		tHandle(null),
-		typeFlags(flags | typeCpp),
+		myTypeFlags(flags | typeCpp),
 		mySize(size) {
 
 		if ((flags & typeValue) == 0)
@@ -99,7 +99,7 @@ namespace storm {
 		engine(RootObject::engine()),
 		myGcType(gcType),
 		tHandle(null),
-		typeFlags(flags | typeCpp),
+		myTypeFlags(flags | typeCpp),
 		mySize(size) {
 
 		myGcType->type = this;
@@ -111,7 +111,7 @@ namespace storm {
 		engine(RootObject::engine()),
 		myGcType(gcType),
 		tHandle(null),
-		typeFlags(flags | typeCpp),
+		myTypeFlags(flags | typeCpp),
 		mySize(size) {
 
 		myGcType->type = this;
@@ -121,7 +121,7 @@ namespace storm {
 	// We need to set myGcType->type first, therefore we call setMyType!
 	Type::Type(Engine &e, TypeFlags flags, Size size, GcType *gcType) :
 		NameSet(setMyType(null, this, gcType, e)), engine(e), myGcType(gcType),
-		tHandle(null), typeFlags(typeClass | typeCpp), mySize(size) {
+		tHandle(null), myTypeFlags(typeClass | typeCpp), mySize(size) {
 
 		init(null);
 	}
@@ -138,7 +138,7 @@ namespace storm {
 	}
 
 	void Type::init(const void *vtable) {
-		assert((typeFlags & typeValue) == typeValue || (typeFlags & typeClass) == typeClass, L"Invalid type flags!");
+		assert((myTypeFlags & typeValue) == typeValue || (myTypeFlags & typeClass) == typeClass, L"Invalid type flags!");
 
 		// Generally, we want this on types.
 		flags |= namedMatchNoInheritance;
@@ -162,7 +162,7 @@ namespace storm {
 			return;
 
 		myVTable = new (engine) VTable(this);
-		if (typeFlags & typeCpp) {
+		if (myTypeFlags & typeCpp) {
 			myVTable->createCpp(vtab);
 		} else if (Type *t = super()) {
 			myVTable->createStorm(t->myVTable);
@@ -187,6 +187,20 @@ namespace storm {
 			myVTable->lateInit();
 	}
 
+	void Type::addTypeFlag(TypeFlags flags) {
+		// Mask out `typeClass` and `typeValue`. Can't change them after creation!
+		flags &= ~typeClass;
+		flags &= ~typeValue;
+
+		// Also, information from C++ can not be added after the fact.
+		flags &= ~typeCpp;
+		flags &= ~typeCppPOD;
+		flags &= ~typeCppSimple;
+
+		// Finally, it is safe to update our flags.
+		myTypeFlags |= flags;
+	}
+
 	// Our finalizer.
 	static void CODECALL destroyType(void *obj, os::Thread *) {
 		Type *t = (Type *)obj;
@@ -261,7 +275,13 @@ namespace storm {
 			return Object::stormType(engine);
 	}
 
-	void Type::setSuper(Type *to) {
+	void Type::setSuper(MAYBE(Type *) to) {
+		if (to && (to->typeFlags() & typeFinal)) {
+			StrBuf *msg = new (engine) StrBuf();
+			*msg << S("Can not inherit from the final type ") << to->identifier() << S(".");
+			throw new (engine) TypedefError(pos, msg->toS());
+		}
+
 		// If 'to' is null: figure out what to inherit from.
 		if (!to)
 			to = defaultSuper();
@@ -270,6 +290,14 @@ namespace storm {
 		if (to == this)
 			return;
 
+		// Check so that values only inherit from values, and classes only from classes.
+		if (to && value() != to->value()) {
+			StrBuf *msg = new (engine) StrBuf();
+			*msg << S("The type ") << identifier() << S(" can not extend the type ")
+				 << to->identifier() << S(" since one is a value, and the other is not.");
+			throw new (engine) TypedefError(pos, msg->toS());
+		}
+
 		if (!chain)
 			chain = new (this) TypeChain(this);
 
@@ -308,7 +336,7 @@ namespace storm {
 		}
 
 		// Invalidate the GcType for this type.
-		if (typeFlags & typeCpp) {
+		if (myTypeFlags & typeCpp) {
 			// Type from C++. Nothing to do.
 		} else if (myGcType != null) {
 			// We're not a type from C++.
@@ -320,7 +348,7 @@ namespace storm {
 		// TODO: Invalidate the Layout as well.
 		// TODO: Invalidate the handle as well.
 
-		if ((typeFlags & typeCpp) != typeCpp && !value()) {
+		if ((myTypeFlags & typeCpp) != typeCpp && !value()) {
 			// Re-initialize the vtable.
 			myVTable->createStorm(to->myVTable);
 		}
@@ -427,7 +455,7 @@ namespace storm {
 				updateHandle(f);
 		}
 
-		if ((typeFlags & typeCpp) != typeCpp) {
+		if ((myTypeFlags & typeCpp) != typeCpp) {
 			// Tell the layout we found a new variable!
 			if (!layout)
 				layout = new (engine) Layout();
@@ -481,7 +509,7 @@ namespace storm {
 		}
 
 		// We need to create the description. Switch threads if neccessary.
-		const os::Thread &t = TObject::thread->thread();
+		const os::Thread &t = TObject::associatedThread()->thread();
 		if (t != os::Thread::current()) {
 			// Switch threads...
 			Type *me = this;
@@ -660,7 +688,7 @@ namespace storm {
 			return mySize;
 
 		// We need to compute our size. Switch threads if neccessary...
-		const os::Thread &t = TObject::thread->thread();
+		const os::Thread &t = TObject::associatedThread()->thread();
 		if (t != os::Thread::current()) {
 			// Switch threads...
 			Type *me = this;
@@ -690,7 +718,7 @@ namespace storm {
 			return myVTable;
 
 		// We need to load ourself. Make sure we're running on the proper thread.
-		const os::Thread &t = TObject::thread->thread();
+		const os::Thread &t = TObject::associatedThread()->thread();
 		if (t != os::Thread::current()) {
 			// Switch threads...
 			Type *me = this;
@@ -721,7 +749,7 @@ namespace storm {
 			return *tHandle;
 
 		// We need to create the handle. Switch threads.
-		const os::Thread &t = TObject::thread->thread();
+		const os::Thread &t = TObject::associatedThread()->thread();
 		if (t != os::Thread::current()) {
 			// Switch threads...
 			Type *me = this;
@@ -754,8 +782,9 @@ namespace storm {
 			if (t->isA(TObject::stormType(t->engine))) {
 				// We might need to switch threads...
 				TObject *obj = (TObject *)object;
-				if (obj->thread) {
-					os::Thread desired = obj->thread->thread();
+				Thread *t = obj->associatedThread();
+				if (t) {
+					os::Thread desired = t->thread();
 					if (desired != os::Thread::current()) {
 						// We can't execute it on this thread. Tell the GC we need a different one!
 						*thread = desired;
@@ -830,7 +859,7 @@ namespace storm {
 			return r;
 
 		// Call on proper thread...
-		const os::Thread &t = TObject::thread->thread();
+		const os::Thread &t = TObject::associatedThread()->thread();
 		if (t != os::Thread::current()) {
 			// Wrong thread, switch!
 			Type *me = this;
@@ -908,7 +937,7 @@ namespace storm {
 			return myGcType;
 
 		// We need to create it. Switch threads if neccessary...
-		const os::Thread &t = TObject::thread->thread();
+		const os::Thread &t = TObject::associatedThread()->thread();
 		if (t != os::Thread::current()) {
 			// Wrong thread, switch!
 			Type *me = this;
@@ -919,7 +948,7 @@ namespace storm {
 		}
 
 		// We're on the correct thread. Compute the type!
-		assert(value() || (typeFlags & typeCpp) != typeCpp, L"C++ types should be given a GcType on creation!");
+		assert(value() || (myTypeFlags & typeCpp) != typeCpp, L"C++ types should be given a GcType on creation!");
 
 
 		// Make sure everything is loaded.
