@@ -1,6 +1,7 @@
 #include "stdafx.h"
 #include "Future.h"
 #include "CloneEnv.h"
+#include "Exception.h"
 #include "Utils/Memory.h"
 
 namespace storm {
@@ -49,6 +50,14 @@ namespace storm {
 		}
 	}
 
+	void FutureBase::error(Exception *exception) {
+		try {
+			exception->throwMe();
+		} catch (...) {
+			error();
+		}
+	}
+
 	static os::PtrThrowable *cloneEx(os::PtrThrowable *src, void *) {
 		if (Object *o = dynamic_cast<Object *>(src)) {
 			return clone(o);
