@@ -12,7 +12,7 @@ namespace storm {
 		class Rect {
 			STORM_VALUE;
 		public:
-			// Empty rectangle.
+			// Create an empty rectangle.
 			STORM_CTOR Rect();
 
 			// Initialize to (0, 0)-(size)
@@ -31,27 +31,30 @@ namespace storm {
 			Point p0;
 			Point p1;
 
-			// Normalized.
+			// Return a normalized rectangle, where `p0` is to the top-left of `p1`.
 			Rect STORM_FN normalized() const;
 
-			// Contains a point?
+			// Does this rectangle contain the point `pt`.
 			Bool STORM_FN contains(Point pt) const;
 
-			// Contains a rectangle?
+			// Does this rectangle contain another rectangle? For this to be true, all parts of
+			// `rect` needs to be inside this rectangle.
 			Bool STORM_FN contains(Rect rect) const;
 
-			// Intersects with another Rect?
+			// Does this rectangle intersect with another rectangle?
 			Bool STORM_FN intersects(Rect other) const;
 
 			// Size.
 			inline Size STORM_FN size() const { return p1 - p0; }
 			inline void STORM_ASSIGN size(Size to) { p1 = p0 + to; }
-			inline Rect STORM_FN sized(Size to) { Rect r = *this; r.size(to); return r; }
 
-			// Center.
+			// Return a rectangle with the same origin, but the specified size.
+			inline Rect STORM_FN sized(Size to) { return Rect(p0, to); }
+
+			// Compute the center of the rectangle.
 			inline Point STORM_FN center() const { return (p0 + p1) / 2; }
 
-			// Move.
+			// Move the origin of the rectangle to `to`.
 			inline Rect STORM_FN at(Point to) const { return Rect(to, size()); }
 
 			Rect STORM_FN operator +(Point pt) const;
@@ -59,21 +62,25 @@ namespace storm {
 			Rect &STORM_FN operator +=(Point pt);
 			Rect &STORM_FN operator -=(Point pt);
 
-			// Include a point in the bounding box.
+			// Increase the rectangle to include the point `to`. Can be seen as a single step in
+			// computing a bounding box of a number of points.
 			Rect STORM_FN include(Point to) const;
 
-			// Include a rectangle. Yields the bounding box of this rectangle and 'other'.
+			// Increase the dimensions of the rectangle to include `other`. Can be seen as the union
+			// of two bounding boxes.
 			Rect STORM_FN include(Rect other) const;
 
 			// Scale the entire rect around its center.
 			Rect STORM_FN scaled(Float scale) const;
 
-			// Shrink/grow according to a size.
+			// Shrink the rectangle in all dimensions based on the height and width in `rate`.
 			Rect STORM_FN shrink(Size rate) const;
+
+			// Grow the rectangle in all dimensions based on the height and width in `rate`.
 			Rect STORM_FN grow(Size rate) const;
 		};
 
-		// Point inside.
+		// Check if the point `pt` is inside the rectangle `r`. Same as `r.contains(pt)`.
 		Bool STORM_FN inside(Point pt, Rect r);
 
 		// ToS.
