@@ -12,6 +12,27 @@ namespace storm {
 
 		Vector::Vector(Point p) : x(p.x), y(p.y), z(0) {}
 
+		Float Vector::lengthSq() const {
+			return *this * *this;
+		}
+
+		Float Vector::length() const {
+			return sqrt(lengthSq());
+		}
+
+		Vector Vector::normalized() const {
+			Float len = length();
+			if (len == 0) {
+				return (*this);
+			} else {
+				return (*this) / len;
+			}
+		}
+
+		Float Vector::taxiLength() const {
+			return ::fabs(x) + ::fabs(y) + ::fabs(z);
+		}
+
 		wostream &operator <<(wostream &to, const Vector &s) {
 			return to << L"(" << s.x << L", " << s.y << L", " << s.z << L")";
 		}
@@ -32,6 +53,20 @@ namespace storm {
 			return Vector(-a.x, -a.y, -a.z);
 		}
 
+		Vector &operator +=(Vector &a, Vector b) {
+			a.x += b.x;
+			a.y += b.y;
+			a.z += b.z;
+			return a;
+		}
+
+		Vector &operator -=(Vector &a, Vector b) {
+			a.x -= b.x;
+			a.y -= b.y;
+			a.z -= b.z;
+			return a;
+		}
+
 		Vector operator *(Vector a, Float b) {
 			return Vector(a.x * b, a.y * b, a.z * b);
 		}
@@ -44,6 +79,20 @@ namespace storm {
 			return Vector(a.x / b, a.y / b, a.z / b);
 		}
 
+		Vector &operator *=(Vector &a, Float b) {
+			a.x *= b;
+			a.y *= b;
+			a.z *= b;
+			return a;
+		}
+
+		Vector &operator /=(Vector &a, Float b) {
+			a.x /= b;
+			a.y /= b;
+			a.z /= b;
+			return a;
+		}
+
 		// Dot and cross product.
 		Float operator *(Vector a, Vector b) {
 			return a.x * b.x + a.y * b.y + a.z * b.z;
@@ -57,5 +106,11 @@ namespace storm {
 			return Vector(::fabs(a.x), ::fabs(a.y), ::fabs(a.z));
 		}
 
+		Vector project(Vector pt, Vector origin, Vector dir) {
+			pt -= origin;
+			float t = (pt * dir) / (dir * dir);
+			return origin + dir * t;
+		}
+
 	}
 }
