@@ -31,7 +31,7 @@ namespace storm {
 	}
 
 	void ProtocolNotSupported::message(StrBuf *to) const {
-		*to << operation << S(" is not supported by the protocol ") << protocol;
+		*to << S("The operation '") << operation << S("' is not supported by the protocol ") << protocol;
 	}
 
 
@@ -65,6 +65,10 @@ namespace storm {
 		throw new (this) ProtocolNotSupported(S("createDir"), toS());
 	}
 
+	Bool Protocol::unlink(Url *url) {
+		throw new (this) ProtocolNotSupported(S("delete"), toS());
+	}
+
 	Str *Protocol::format(Url *url) {
 		throw new (this) ProtocolNotSupported(S("format"), toS());
 	}
@@ -116,6 +120,23 @@ namespace storm {
 		*to << S("./");
 	}
 
+	Str *RelativeProtocol::format(Url *url) {
+		StrBuf *to = new (this) StrBuf();
+
+		Array<Str *> *parts = url->getParts();
+		if (parts->count() == 0)
+			return to->toS();
+
+		*to << parts->at(0);
+		for (Nat i = 1; i < parts->count(); i++)
+			*to << S("/") << parts->at(i);
+
+		if (url->dir())
+			*to << L"/";
+
+		return to->toS();
+	}
+
 	SerializedType *RelativeProtocol::serializedType(EnginePtr e) {
 		SerializedStdType *type = serializedStdType<RelativeProtocol, Protocol>(e.v);
 		// No data.
@@ -257,6 +278,18 @@ namespace storm {
 		return CreateDirectory(format(url)->c_str(), NULL) == TRUE;
 	}
 
+	Bool FileProtocol::unlink(Url *url) {
+		Str *file = format(url);
+		DWORD attrs = GetFileAttributes(file->c_str());
+		if (attrs == INVALID_FILE_ATTRIBUTES)
+			return false;
+
+		if (attrs & FILE_ATTRIBUTE_DIRECTORY)
+			return RemoveDirectory(file->c_str()) != FALSE;
+		else
+			return DeleteFile(file->c_str()) != FALSE;
+	}
+
 #elif defined(POSIX)
 
 	Bool FileProtocol::partEq(Str *a, Str *b) {
@@ -333,6 +366,19 @@ namespace storm {
 		return mkdir(format(url)->utf8_str(), 0777) == 0;
 	}
 
+	Bool FileProtocol::unlink(Url *url) {
+		const char *name = format(url)->utf8_str();
+
+		struct stat s;
+		if (::stat(name, &s) != 0)
+			return false;
+
+		if (S_ISDIR(s.st_mode))
+			return ::rmdir(name) == 0;
+		else
+			return ::unlink(name) == 0;
+	}
+
 #else
 #error "Please implement FileProtocol for your OS!"
 #endif
