@@ -87,6 +87,9 @@ namespace storm {
 		// Create a directory.
 		virtual Bool STORM_FN createDir(Url *url);
 
+		// Delete a file or directory.
+		virtual Bool STORM_FN STORM_NAME(unlink,delete)(Url *url);
+
 		// Convert to a string suitable for other C-api:s
 		virtual Str *STORM_FN format(Url *url);
 
@@ -124,6 +127,9 @@ namespace storm {
 		// Output.
 		virtual void STORM_FN toS(StrBuf *to) const;
 
+		// Formatting.
+		virtual Str *STORM_FN format(Url *url);
+
 		// Serialization:
 		static SerializedType *STORM_FN serializedType(EnginePtr e);
 		static RelativeProtocol *STORM_FN read(ObjIStream *from);
@@ -162,6 +168,9 @@ namespace storm {
 		// Create a directory.
 		virtual Bool STORM_FN createDir(Url *url);
 
+		// Delete a file or directory.
+		virtual Bool STORM_FN STORM_NAME(unlink,delete)(Url *url);
+
 		// Convert an Url to a string suitable for other C-api:s.
 		virtual Str *STORM_FN format(Url *url);
 
