@@ -56,7 +56,7 @@ namespace storm {
 #endif
 
 	static GcArray<wchar> empty = {
-		1,
+		1, 0, { 0 }
 	};
 
 	Str::Str() : data(&storm::empty) {}
@@ -64,8 +64,11 @@ namespace storm {
 	Str::Str(const wchar *s) {
 		nat count = nat(wcslen(s));
 		allocData(count + 1);
-		for (nat i = 0; i < count; i++)
+		for (nat i = 0; i < count; i++) {
 			data->v[i] = s[i];
+			if (utf16::leading(s[i]))
+				data->filled++;
+		}
 		data->v[count] = 0;
 		validate();
 	}
@@ -73,6 +76,10 @@ namespace storm {
 #ifdef POSIX
 	Str::Str(const wchar_t *s) {
 		data = toWChar(engine(), s);
+		// Count surrogate pairs:
+		for (Nat i = 0; i < data->count; i++)
+			if (utf16::leading(data->v[i]))
+				data->filled++;
 		validate();
 	}
 #endif
@@ -80,23 +87,29 @@ namespace storm {
 	Str::Str(const wchar *from, const wchar *to) {
 		nat count = nat(to - from);
 		allocData(count + 1);
-		for (nat i = 0; i < count; i++)
+		for (nat i = 0; i < count; i++) {
 			data->v[i] = from[i];
+			if (utf16::leading(from[i]))
+				data->filled++;
+		}
 		data->v[count] = 0;
 		validate();
 	}
 
-	static inline void copy(wchar *&to, const wchar *begin, const wchar *end) {
-		for (const wchar *at = begin; at != end; at++)
+	static inline void copy(wchar *&to, const wchar *begin, const wchar *end, size_t &leading) {
+		for (const wchar *at = begin; at != end; at++) {
 			*(to++) = *at;
+			if (utf16::leading(*at))
+				leading++;
+		}
 	}
 
 	Str::Str(const wchar *fromA, const wchar *toA, const wchar *fromB, const wchar *toB) {
 		nat count = nat((toA - fromA) + (toB - fromB));
 		allocData(count + 1);
 		wchar *to = data->v;
-		copy(to, fromA, toA);
-		copy(to, fromB, toB);
+		copy(to, fromA, toA, data->filled);
+		copy(to, fromB, toB, data->filled);
 		*to = 0;
 		validate();
 	}
@@ -109,9 +122,9 @@ namespace storm {
 		const wchar *first = src->data->v;
 		const wchar *cut = src->toPtr(pos);
 		const wchar *last = first + src->charCount();
-		copy(to, first, cut);
-		copy(to, insert->data->v, insert->data->v + insert->charCount());
-		copy(to, cut, last);
+		copy(to, first, cut, data->filled);
+		copy(to, insert->data->v, insert->data->v + insert->charCount(), data->filled);
+		copy(to, cut, last, data->filled);
 		*to = 0;
 		validate();
 	}
@@ -124,6 +137,7 @@ namespace storm {
 			allocData(3);
 			data->v[0] = lead;
 			data->v[1] = trail;
+			data->filled = 1;
 		} else if (trail) {
 			allocData(2);
 			data->v[0] = trail;
@@ -143,6 +157,7 @@ namespace storm {
 				data->v[i*2] = lead;
 				data->v[i*2 + 1] = trail;
 			}
+			data->filled = times;
 		} else if (trail) {
 			allocData(times + 1);
 			for (nat i = 0; i < times; i++) {
@@ -155,17 +170,25 @@ namespace storm {
 	}
 
 	Str::Str(GcArray<wchar> *data) : data(data) {
+		// Count surrogate pairs:
+		for (Nat i = 0; i < data->count; i++)
+			if (utf16::leading(data->v[i]))
+				data->filled++;
 		validate();
 	}
 
 	void Str::validate() const {
 #ifdef SLOW_DEBUG
+		Nat surrogates = 0;
 		for (nat i = 0; i < data->count - 1; i++) {
 			if (data->v[i] == 0) {
 				assert(false, L"String contains a premature null terminator!");
+			} else if (utf16::leading(data->v[i])) {
+				surrogates++;
 			}
 		}
 		assert(data->v[data->count - 1] == 0, L"String is missing a null terminator!");
+		assert(data->filled == surrogates, L"Number of surrogates is incorrect.");
 #endif
 	}
 
@@ -177,6 +200,10 @@ namespace storm {
 		return !empty();
 	}
 
+	Nat Str::count() const {
+		return Nat(data->count - data->filled - 1);
+	}
+
 	Str *Str::operator +(Str *o) const {
 		return new (this) Str(this, o);
 	}
@@ -383,6 +410,13 @@ namespace storm {
 		return -1;
 	}
 
+	Bool Str::isHex() const {
+		for (Nat i = 0; i < data->count - 1; i++)
+			if (hexDigit(data->v[i]) < 0)
+				return false;
+		return true;
+	}
+
 	Nat Str::hexToNat() const {
 		return Nat(hexToWord());
 	}
@@ -758,20 +792,105 @@ namespace storm {
 		return pos;
 	}
 
+	Str::Iter Str::find(Str *str) const {
+		return find(str, begin());
+	}
+
+	Str::Iter Str::find(Str *str, Iter start) const {
+		Iter sFirst = str->begin();
+		Iter sLast = str->end();
+		Char firstCh = sFirst.v();
+		++sFirst;
+
+		Iter last = end();
+		for (Iter pos = start; pos != last; ++pos) {
+			if (pos.v() == firstCh) {
+				Bool ok = true;
+
+				// Check the remainder of the strings.
+				for (Iter i = sFirst, j = pos + 1; i != sLast; i++, j++) {
+					if (j == last || i.v() != j.v()) {
+						ok = false;
+						break;
+					}
+				}
+
+				// Did we find it?
+				if (ok)
+					return pos;
+			}
+		}
+
+		return last;
+	}
+
 	Str::Iter Str::findLast(Char ch) const {
 		return findLast(ch, end());
 	}
 
 	Str::Iter Str::findLast(Char ch, Iter last) const {
-		// TODO: We should search backwards...
-		Iter pos = begin();
-		Iter result = end();
-		for (; pos != last; ++pos) {
-			if (pos.v() == ch)
-				result = pos;
+		if (!data)
+			return end();
+		if (last.owner && last.owner != this)
+			return end();
+
+		wchar leading = ch.leading();
+		wchar trailing = ch.trailing();
+		if (leading == 0) {
+			leading = trailing;
+			trailing = 0;
 		}
 
-		return result;
+		Nat endPos = Nat(data->count - 1);
+		if (!last.atEnd())
+			endPos = std::min(endPos, last.pos);
+
+		const wchar *start = data->v;
+		for (const wchar *at = data->v + endPos; at > start; at--) {
+			const wchar check = at[-1];
+			if (check == leading) {
+				if (trailing == 0 || trailing == at[0])
+					return Iter(this, Nat(at - start - 1));
+			}
+		}
+
+		return end();
+	}
+
+	Str::Iter Str::findLast(Str *str) const {
+		return findLast(str, end());
+	}
+
+	Str::Iter Str::findLast(Str *str, Iter last) const {
+		if (!data)
+			return end();
+		if (last.owner && last.owner != this)
+			return end();
+
+		Nat endPos = Nat(data->count);
+		if (!last.atEnd())
+			endPos = std::min(endPos, last.pos);
+
+		const wchar *strStart = str->data->v;
+		const wchar *strEnd = strStart + str->data->count - 1;
+		const wchar *thisStart = data->v;
+		for (const wchar *at = data->v + endPos; at > thisStart; at--) {
+			if (at[-1] == strEnd[-1]) {
+				bool ok = true;
+
+				for (const wchar *other = strEnd - 1, *me = at - 1; other > strStart; other--, me--) {
+					if (me <= thisStart || me[-1] != other[-1]) {
+						ok = false;
+						break;
+					}
+				}
+
+				if (ok)
+					return Iter(this, Nat(at - thisStart - (strEnd - strStart)));
+			}
+		}
+
+		return end();
 	}
 
 	void Str::write(OStream *to) const {
@@ -937,12 +1056,18 @@ namespace storm {
 	}
 
 	Bool Str::Iter::operator >(const Iter &o) const {
-		return o < *this;
+		if (!o.atEnd() && atEnd())
+			return true;
+
+		if (owner != o.owner)
+			return false;
+
+		return pos > o.pos;
 	}
 
 	Bool Str::Iter::operator <(const Iter &o) const {
 		if (o.atEnd() && !atEnd())
-			return false;
+			return true;
 
 		if (owner != o.owner)
 			return false;
@@ -1151,9 +1276,13 @@ namespace storm {
 
 	StrBuf *operator <<(StrBuf *to, Str::Iter iter) {
 		*to << S("Iterator: ");
-		*to << iter.data()->substr(iter.data()->begin(), iter);
-		*to << S("|>");
-		*to << iter.data()->substr(iter);
+		if (const Str *data = iter.data()) {
+			*to << data->substr(data->begin(), iter);
+			*to << S("|>");
+			*to << data->substr(iter);
+		} else {
+			*to << S("<none>");
+		}
 		return to;
 	}
 
