@@ -25,6 +25,7 @@
 #include "Format.h" // for fmt::wordAlign
 #include "SampleImpl.h"
 #include "Root.h"
+#include "ExceptionHandler.h"
 
 #ifdef STORM_GC
 
@@ -149,11 +150,24 @@ namespace storm {
 			return impl->allocArray(type, count);
 		}
 
+		// Allocate an array of objects in response to a stale location dependency that requires us
+		// to re-hash a hash table.
+		inline void *allocArrayRehash(const GcType *type, size_t count) {
+			assert(type->kind == GcType::tArray, L"Wrong type for calling allocArray().");
+
+			return impl->allocArrayRehash(type, count);
+		}
+
 		// Allocate an array of weak pointers.
 		inline void *allocWeakArray(size_t count) {
 			return impl->allocWeakArray(&weakArrayType, count);
 		}
 
+		// Allocate a weak array in response to a stale location dependency.
+		inline void *allocWeakArrayRehash(size_t count) {
+			return impl->allocWeakArrayRehash(&weakArrayType, count);
+		}
+
 		// See if the object is live. An object is considered live until it has been
 		// finalized. Finalized objects may not be collected immediately after they have been
 		// finalized, and therefore they may still appear inside weak sets etc. after that. The GC
@@ -303,6 +317,22 @@ namespace storm {
 		}
 
 
+		/**
+		 * Exception handling.
+		 *
+		 * On some platforms, the Gc needs to set up callbacks based on allocated memory. This is
+		 * where to hook into these callbacks.
+		 */
+
+#ifdef STORM_GC_EH_CALLBACK
+		inline void setEhCallback(STORM_GC_EH_CALLBACK callback) {
+			ehCallback = callback;
+		}
+
+		static STORM_GC_EH_CALLBACK getEhCallback(GcImpl *impl);
+#endif
+
+
 		/**
 		 * Debugging/testing.
 		 */
@@ -353,6 +383,10 @@ namespace storm {
 		// Lock for the root set.
 		util::Lock rootLock;
 
+#ifdef STORM_GC_EH_CALLBACK
+		STORM_GC_EH_CALLBACK ehCallback;
+#endif
+
 		// Destroyed already?
 		Bool destroyed;
 	};
