@@ -904,10 +904,129 @@ namespace os {
 		return address(&doEndDetour);
 	}
 
+#elif defined(VISUAL_STUDIO) && defined(X64)
+
+	void UThreadData::initStack() {
+		// Not necessary on X64.
+	}
+
+	void UThreadData::updateOwner(UThreadState *state) {
+		// Not necessary on X64.
+		(void)state;
+	}
+
+	static void *&stackPtr(Stack &stack) {
+		return stack.desc->low;
+	}
+
+	void UThreadData::push(void *v) {
+		void **esp = (void **)stackPtr(stack);
+		*--esp = v;
+		stackPtr(stack) = esp;
+	}
+
+	void UThreadData::push(uintptr_t v) {
+		void **esp = (void **)stackPtr(stack);
+		*--esp = (void *)v;
+		stackPtr(stack) = esp;
+	}
+
+	void UThreadData::push(intptr_t v) {
+		void **esp = (void **)stackPtr(stack);
+		*--esp = (void *)v;
+		stackPtr(stack) = esp;
+	}
+
+	void UThreadData::pushContext(const void *fn) {
+		pushContext(fn, null);
+	}
+
+	extern "C" void doStartThread();
+
+	void UThreadData::pushContext(const void *fn, void *param) {
+		// Allocate shadow space for register parameters, in case the first function needs it.
+		for (size_t i = 0; i < 4; i++)
+			push((void *)0);
+
+		push((void *)doStartThread); // Return to.
+		push((void *)0); // rbp
+		push((void *)fn); // rbx (call function)
+		push((void *)param); // rdi (parameter)
+		push((void *)0); // rsi (simulated return ptr)
+		push((void *)0); // r12
+		push((void *)0); // r13
+		push((void *)0); // r14
+		push((void *)0); // r15
+
+		push((void *)0); // alignment
+
+		for (size_t i = 0; i < 10; i++) {
+			push((void *)0); // XMM register
+			push((void *)0); // 2 words
+		}
+
+		push((void *)0); // alignment
+
+		push(stack.desc->high);
+		push(stack.desc->dummy);
+		push(stack.desc->low);
+
+		// Note: The asm function does have shadow space for registers allocated, but we compensate
+		// for that when allocating a sub context. Otherwise the pointer to our created desc will be
+		// wrong.
+
+		// Set the 'desc' of 'stack' to the actual stack pointer, so that we can run code on this stack.
+		atomicWrite(stack.desc, (StackDesc *)(stack.desc->low));
+	}
+
+	// Logical location where thread switches are made. Used to get good backtraces.
+	// Note: This is a label in asm, so the interesting thing about this is its address.
+	extern "C" char doSwitchReturnLoc;
+
+	StackDesc *UThreadData::pushSubContext(const void *fn, void *param) {
+		StackDesc *old = stack.desc;
+		const size_t regCount = 34; // 8 general regs, return ptr, 10*2 xmm regs, 3 desc, 2 alignment
+
+		// Copy the old context to the top of the stack again.
+		// Note: the ASM code does not account for shadow space, so we need to compensate here.
+		size_t *oldStack = (size_t *)old;
+		size_t *newStack = oldStack - regCount - 4; // 4 is for shadow space
+		memcpy(newStack, oldStack, sizeof(void *) * regCount);
+
+		// Update the copy:
+		newStack[regCount - 1] = size_t(&doStartThread); // return to, use our thunk
+		newStack[regCount - 3] = size_t(fn); // rbx, function to call
+		newStack[regCount - 4] = size_t(param); // rdi, parameter
+		newStack[regCount - 5] = size_t(&doSwitchReturnLoc); // return pointer when called, used for stack traces
+
+		// Install the new frame.
+		atomicWrite(stack.desc, (StackDesc *)(newStack));
+
+		return old;
+	}
+
+	void UThreadData::restoreContext(StackDesc *desc) {
+		atomicWrite(stack.desc, (StackDesc *)desc);
+	}
+
+	// Called from UThreadX64.asm64
+	extern "C" void doEndDetour2(void **result) {
+		UThreadState::current()->endDetour(result);
+	}
+
+	extern "C" void doEndDetour();
+
+	static const void *endDetourFn(bool member) {
+		// Note: We don't need to do anything special for member functions. MSVC uses thunks for
+		// vtable lookups.
+		(void)member;
+		return address(doEndDetour);
+	}
+
 #elif defined(GCC) && defined(X64)
 
 	void UThreadData::initStack() {
-		// Not necessary on ARM.
+		// Not necessary on X64.
 	}
 
 	void UThreadData::updateOwner(UThreadState *state) {
