@@ -9,24 +9,38 @@ namespace sound {
 	 * avoid heap allocations (at least from C++).
 	 *
 	 * If multiple copies of a buffer are created, they all refer to the same backing storage,
-	 * slightly breaking the memory model of Storm.
-	 *
-	 * TODO: Make proper element access operators from Storm.
+	 * slightly breaking the memory model of Storm, as a buffer is a value that behaves like an
+	 * object. This allows the `Buffer` to work properly in multithreaded operations with some care.
+	 * Interfaces typically need to both accept and return a buffer to work properly with threads.
+	 * If a thread-switch is not needed, then the same buffer can be returned which results in no
+	 * copies being made. If a thread-switch is made, the buffer is copied, and the returned value
+	 * with up-to-date information is used. See `core.io.IStream` for an example of this.
 	 */
 	class Buffer {
 		STORM_VALUE;
 	public:
-		// Default constructor. Empty buffer.
+		// Default constructor. Create an empty buffer. Use the function `buffer` to create a buffer
+		// with non-zero size.
 		STORM_CTOR Buffer();
 
-		// Number of bytes in total.
+		// Get the number of samples in the buffer in total.
 		inline Nat STORM_FN count() const { return data ? Nat(data->count) : 0; }
 
-		// Number of bytes containing valid data. Constrains 'filled' to not be larger than 'count'.
+		// Get the number of samples from the beginning of the buffer that are considered to contain
+		// valid data. The buffer itself generally does not use this value. Rather, it is used as an
+		// universal marker for how much of the buffer is used for actual data, and how much is
+		// free.
 		inline Nat STORM_FN filled() const { return data ? Nat(data->filled) : 0; }
+
+		// Set the number of samples that are used. Limits the value to be maximum `count`. Marked
+		// as an assign functions, so that `filled` can be used as a member variable in languages
+		// like Basic Storm.
 		inline void STORM_FN filled(Nat p) { if (data) data->filled = min(p, count()); }
 
-		// Element access.
+		// Get the number of free samples in the buffer, based on the `filled` member.
+		inline Nat STORM_FN free() const { return count() - filled(); }
+
+		// Access individual samples.
 		Float &STORM_FN operator [](Nat id) { return data->v[id]; }
 		Float operator [](Nat id) const { return data->v[id]; }
 
@@ -36,8 +50,13 @@ namespace sound {
 		// Get a pointer to the data.
 		Float *dataPtr() { return data ? data->v : null; }
 
-		// Empty/full?
+		// Is the buffer empty according to the `filled` member?
 		inline Bool STORM_FN empty() const { return filled() == 0; }
+
+		// Is there any data in the buffer, according to the `filled` member?
+		inline Bool STORM_FN any() const { return filled() > 0; }
+
+		// Is the buffer full, according to the `filled` member?
 		inline Bool STORM_FN full() const { return filled() == count(); }
 
 	private:
@@ -53,7 +72,7 @@ namespace sound {
 		friend Buffer buffer(EnginePtr e, const Float *data, Nat count);
 	};
 
-	// Create a buffer.
+	// Create a buffer with room for `count` bytes. It will be empty initially.
 	Buffer STORM_FN buffer(EnginePtr e, Nat count);
 
 	// Create a buffer using a (potentially stack-allocated) GcArray for backing data. This function
@@ -72,7 +91,9 @@ namespace sound {
 	Buffer cut(EnginePtr e, Buffer src, Nat from);
 	Buffer cut(EnginePtr e, Buffer src, Nat from, Nat count);
 
-	// Output.
+	// Output the buffer, specifying an additional mark.
 	void STORM_FN outputMark(StrBuf &to, Buffer b, Nat markAt);
+
+	// Output the buffer. The system generates a `toS` function as well.
 	StrBuf &STORM_FN operator <<(StrBuf &to, Buffer b);
 }
