@@ -389,6 +389,21 @@ BEGIN_TEST(ExceptionCatch, Code) {
 	Ref appendFn = arena->external(S("appendFn"), address(&::addBuf));
 	Ref freeInt = arena->external(S("freeInt"), address(&::intCleanupGc));
 
+	// Separate function called.
+	// This function ensures that we call destructors even when the return address points to the
+	// start of the function epilog. This was an issue in the 64-bit Windows backend earlier.
+	Listing *shim = new (e) Listing();
+	Var p = shim->createParam(intDesc(e));
+	Var w = shim->createVar(shim->root(), Size::sInt, freeInt, freeOnException);
+	*shim << prolog();
+	*shim << mov(w, intConst(10));
+	*shim << fnParam(intDesc(e), p);
+	*shim << fnCall(errorFn, false);
+	*shim << fnRet();
+
+	Binary *bShim = new (e) Binary(arena, shim);
+
+
 	Listing *l = new (e) Listing(false, ptrDesc(e));
 
 	Var tmp = l->createVar(l->root(), Size::sPtr);
@@ -409,7 +424,9 @@ BEGIN_TEST(ExceptionCatch, Code) {
 	*l << begin(block);
 	*l << mov(innerInt, intConst(4));
 	*l << fnParam(intDesc(e), param);
-	*l << fnCall(errorFn, false);
+	// *l << fnCall(errorFn, false);
+	RefSource *rs = new (e) StrRefSource(new (e) Str(S("errorShim")), bShim);
+	*l << fnCall(Ref(rs), false);
 	*l << end(block);
 	*l << fnRet(ptrConst(0));
 
@@ -431,7 +448,6 @@ BEGIN_TEST(ExceptionCatch, Code) {
 	*l << sub(outerInt, intConst(1));
 	*l << fnRet(tmp);
 
-
 	Binary *b = new (e) Binary(arena, l);
 	typedef Object *(*Fn)(Int i);
 	Fn fn = (Fn)b->address();
@@ -453,25 +469,25 @@ BEGIN_TEST(ExceptionCatch, Code) {
 	destroyed = 0;
 	CHECK_EQ(::toS((*fn)(1)), L"Throw me!");
 	CHECK(DbgVal::clear());
-	CHECK_EQ(destroyed, 4 + 7);
+	CHECK_EQ(destroyed, 4 + 7 + 10);
 
 	// Throws a StrBuf at first, then catches it as a StrBuf and adds an!
 	destroyed = 0;
 	CHECK_EQ(::toS((*fn)(10)), L"Buffer!");
 	CHECK(DbgVal::clear());
-	CHECK_EQ(destroyed, 4 + 17);
+	CHECK_EQ(destroyed, 4 + 17 + 10);
 
 	// Throws a StrBuf, catches it, and throws another exception.
 	destroyed = 0;
 	CHECK_ERROR(::toS((*fn)(2)), Str *);
 	CHECK(DbgVal::clear());
-	CHECK_EQ(destroyed, 4 + 18);
+	CHECK_EQ(destroyed, 4 + 18 + 10);
 
 	// Throws a const Str and tries to catch it.
 	destroyed = 0;
 	CHECK_EQ(::toS((*fn)(20)), L"Throw const!");
 	CHECK(DbgVal::clear());
-	CHECK_EQ(destroyed, 4 + 7);
+	CHECK_EQ(destroyed, 4 + 7 + 10);
 
 	// // DebugBreak();
 	// try {
