@@ -185,14 +185,19 @@
 #define CODECALL
 #endif
 
-#if defined(GCC) && (defined(X64) || defined(ARM64))
-// X86-64 and ARM64 does not need to specify a calling convention. There is a single standard convention!
+#if defined(GCC) && defined(X64)
+// X86-64 does not need to specify a calling convention. There is a single standard convention!
 
 // We're using -falign-functions=2, but that seems to be ignored for static template functions on
 // GCC 8.1.0, so we specify it here as well to be safe.
 #define CODECALL  __attribute__((aligned(2)))
 #endif
 
+#if defined(GCC) && defined(ARM64)
+// ARM64 does not need to specify a calling convention. There is a single standard convention!
+#define CODECALL
+#endif
+
 // Make sure it is defined.
 #ifndef CODECALL
 #error "Someone forgot to declare CODECALL for your architecture."
